/***************************************************************************
*   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#include "ipv4_tcpserver.h"

#include "ipv4_tcp.h"

#include <conexus/utility.h>

namespace Conexus
{

  namespace IPv4
  {

    TCPServer::TCPServer( uint32_t address, uint16_t port )
    {
      m_tcp_listener = Socket::create( PF_INET, SOCK_STREAM, IPPROTO_TCP );
      m_local_address.set_address( address );
      m_local_address.set_port( port );
    }

    TCPServer::TCPServer( const std::string& address, uint16_t port )
    {
      m_tcp_listener = Socket::create( PF_INET, SOCK_STREAM, IPPROTO_TCP );
      m_local_address.set_address( address );
      m_local_address.set_port( port );
    }

    TCPServer::pointer TCPServer::create( uint16_t port )
    {
      return pointer( new TCPServer( INADDR_ANY, port ) );
    }

    TCPServer::pointer TCPServer::create( uint32_t address, uint16_t port )
    {
      return pointer( new TCPServer( address, port ) );
    }

    TCPServer::pointer TCPServer::create( const std::string& address, uint16_t port )
    {
      return pointer( new TCPServer( address, port ) );
    }

    TCPServer::~TCPServer()
    {}

    Address& TCPServer::local_address()
    {
      return m_local_address;
    }

    void TCPServer::set_local_address( Address addr )
    {
      m_local_address = addr;
    }

    void TCPServer::start( )
    {
      this->start( use_dispatcher_default() );
    }

    void TCPServer::start( bool use_dispatcher )
    {
      m_tcp_listener->bind( m_local_address );
      m_tcp_listener->listen( 5 );
      Conexus::EndpointServer::start( use_dispatcher );
    }

    void TCPServer::stop()
    {
      Conexus::EndpointServer::stop();
      m_tcp_listener->close();
    }

    void TCPServer::service_thread_main()
    {
      int n;
      struct timeval tv;
      fd_set readfd;
      int fd;
      int new_connection;
      struct sockaddr addr;
      socklen_t addrlen;
      TCP::pointer newtcp;

      fd = m_tcp_listener->fd();

      if ( fd < 0 )
        throw;

      while ( !m_terminate ) {
        FD_ZERO( &readfd );
        FD_SET( fd, &readfd );

        if ( m_timeval.tv_usec <= 0 )
          n = select( fd + 1, &readfd, NULL, NULL, NULL );
        else {
          tv = m_timeval;
          n = select( fd + 1, &readfd, NULL, NULL, &tv );
        }
        if ( FD_ISSET( fd, &readfd ) ) {
          new_connection = ::accept( fd, &addr, &addrlen );
          if ( new_connection >= 0 ) {
            newtcp = TCP::create( new_connection, true );
            this->queue_endpoint( newtcp );
          } else {
            //TODO put in proper error handling and throws
            std::cout << "Accept failed" << std::endl;
            throw;
          }
        }
      }
      m_running = false;
    }


  } // namespace IPv4

} // namespace Conexus
