/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "utility.h"

#include <netinet/in.h>
#include <sys/ioctl.h>
#include <net/if.h>

// #include <netpacket/packet.h>
// #include <net/ethernet.h>
// #include <netdb.h>
// #include <sys/socket.h>
// #include <netinet/in.h>
// #include <arpa/inet.h>

#include <glibmm/thread.h>

#include <conexus/ipv4_udp.h>
#include <conexus/ipv6_udp.h>

namespace Conexus
{

  bool global_use_dispatcher_default = false;
  bool global_initialized = false;

  unsigned interface_index( const std::string& name )
  {
    unsigned index = if_nametoindex( name.c_str() );
    if ( index == 0 )
      throw interface_index_exception();
    return index;
  }

  std::string interface_name( unsigned ifindex )
  {
    char index_name[IF_NAMESIZE];

    if ( index_name != if_indextoname( ifindex, index_name ) )
      throw interface_index_exception();

    return std::string( index_name );
  }

  std::vector<std::pair<unsigned,std::string> > interfaces()
  {
    struct if_nameindex *index, *i;
    std::vector<std::pair<unsigned,std::string> > ifs;

    index = if_nameindex();
    for ( i = index; i != NULL; i++ )
      ifs.push_back( std::pair<unsigned,std::string>(i->if_index, i->if_name) );
    if_freenameindex(index);

    return ifs;
  }

  std::vector<std::string> interface_names()
  {
    struct if_nameindex *index, *i;
    std::vector<std::string> ifs;

    index = if_nameindex();
    for ( i = index; i != NULL; i++ )
      ifs.push_back( std::string(i->if_name) );
    if_freenameindex(index);

    return ifs;
  }

#define MAX_REQ_SIZE 100

  std::vector< std::pair < std::string , std::string > > interface_ip_addresses( )
  {
    std::vector< std::pair<std::string, std::string> > ifaddrs;

    int sd = socket( PF_PACKET, SOCK_RAW, 0 );
    struct ifreq* req;
    struct ifconf conf;
    char buf[ sizeof( struct ifreq ) * MAX_REQ_SIZE ];
    std::pair<std::string, std::string> interface;
    struct sockaddr_in* sa;

    conf.ifc_len = sizeof( buf );
    conf.ifc_buf = ( caddr_t ) buf;

    if ( ioctl( sd, SIOCGIFCONF, ( caddr_t ) & conf ) >= 0 )
      for ( req = ( struct ifreq* ) conf.ifc_buf; ( caddr_t ) req < conf.ifc_buf + conf.ifc_len; req++ ) {
        interface.first = std::string( req->ifr_name );
        if ( req->ifr_addr.sa_family == AF_INET ) {
          sa = ( struct sockaddr_in* ) & ( req->ifr_addr );
          interface.second = std::string( inet_ntoa( sa->sin_addr ) );
          ifaddrs.push_back( interface );
        }
      }

    close( sd );
    return ifaddrs;
  }

  void init( bool use_dispatcher_default )
  {

    global_use_dispatcher_default = use_dispatcher_default;
    if ( !Glib::thread_supported() )
      Glib::thread_init();

    global_initialized = true;
  }

  bool initialized()
  {
    return global_initialized;
  }

  bool use_dispatcher_default( )
  {
    return global_use_dispatcher_default;
  }

  void set_use_dispatcher_default( bool use_dispatcher_default )
  {
    global_use_dispatcher_default = use_dispatcher_default;
  }

  void set_default_multicast_hops( unsigned hops )
  {
    IPv4::UDP::set_default_multicast_hops( hops );
    IPv6::UDP::set_default_multicast_hops( hops );
  }

  void set_default_multicast_loopback( bool loopback )
  {
    IPv4::UDP::set_default_multicast_loopback( loopback );
    IPv6::UDP::set_default_multicast_loopback( loopback );
  }

}

